// Copyright 2022 The GithubEvents Authors. All rights reserved.
// Use of this source code is governed by the MIT License
// that can be found in the LICENSE file.

package githubevents

// THIS FILE IS GENERATED - DO NOT EDIT DIRECTLY
// make edits in gen/generate.go

import (
	"fmt"
	"github.com/google/go-github/v43/github"
	"golang.org/x/sync/errgroup"
)

// Actions are used to identify registered callbacks.
const (
	// MetaEvent is the event name of github.MetaEvent's
	MetaEvent = "meta"

	// MetaEventAnyAction is used to identify callbacks
	// listening to all events of type github.MetaEvent
	MetaEventAnyAction = "*"
)

// MetaEventHandleFunc represents a callback function triggered on github.MetaEvent.
// deliveryID (type: string) is the unique webhook delivery ID.
// eventName (type: string) is the name of the event.
// event (type: *github.MetaEvent) is the webhook payload.
type MetaEventHandleFunc func(deliveryID string, eventName string, event *github.MetaEvent) error

// OnMetaEventAny registers callbacks listening to events of type github.MetaEvent
//
// This function appends the callbacks passed as arguments to already existing ones.
// If already existing callbacks are to be overwritten, SetOnMetaEventAny must be used.
//
// Callbacks are executed in parallel. This function blocks until all callbacks executed in parallel have returned,
// then returns the first non-nil error (if any) from them. If OnError callbacks have been set, they will be called when an error occurs.
func (g *EventHandler) OnMetaEventAny(callbacks ...MetaEventHandleFunc) {
	g.mu.Lock()
	defer g.mu.Unlock()
	if callbacks == nil || len(callbacks) == 0 {
		panic("callbacks is nil or empty")
	}
	if g.onMetaEvent == nil {
		g.onMetaEvent = make(map[string][]MetaEventHandleFunc)
	}
	g.onMetaEvent[MetaEventAnyAction] = append(
		g.onMetaEvent[MetaEventAnyAction],
		callbacks...,
	)
}

// SetOnMetaEventAny registers callbacks listening to events of type github.MetaEvent
// and overwrites already registered callbacks.
//
// This function overwrites all previously registered callbacks.
// If already registered callbacks are not to be overwritten, OnMetaEventAny must be used.
//
// Callbacks are executed in parallel. This function blocks until all callbacks executed in parallel have returned,
// then returns the first non-nil error (if any) from them. If OnError callbacks have been set, they will be called when an error occurs.
func (g *EventHandler) SetOnMetaEventAny(callbacks ...MetaEventHandleFunc) {
	g.mu.Lock()
	defer g.mu.Unlock()
	if callbacks == nil || len(callbacks) == 0 {
		panic("callbacks is nil or empty")
	}
	if g.onMetaEvent == nil {
		g.onMetaEvent = make(map[string][]MetaEventHandleFunc)
	}
	g.onMetaEvent[MetaEventAnyAction] = callbacks
}

func (g *EventHandler) handleMetaEventAny(deliveryID string, eventName string, event *github.MetaEvent) error {
	if event == nil {
		return fmt.Errorf("event was empty or nil")
	}
	if _, ok := g.onMetaEvent[MetaEventAnyAction]; !ok {
		return nil
	}
	eg := new(errgroup.Group)
	for _, h := range g.onMetaEvent[MetaEventAnyAction] {
		handle := h
		eg.Go(func() error {
			err := handle(deliveryID, eventName, event)
			if err != nil {
				return err
			}
			return nil
		})
	}
	if err := eg.Wait(); err != nil {
		return err
	}
	return nil
}

// MetaEvent handles github.MetaEvent
//
// Callbacks are executed in the following order:
//
// 1) All callbacks registered with OnBeforeAny are executed in parallel.
// 3) All callbacks registered with OnMetaEvent... are executed in parallel in case the Event has actions.
// 4) All callbacks registered with OnAfterAny are executed in parallel.
//
// on any error all callbacks registered with OnError are executed in parallel.
func (g *EventHandler) MetaEvent(deliveryID string, eventName string, event *github.MetaEvent) error {

	if event == nil {
		return fmt.Errorf("event action was empty or nil")
	}

	err := g.handleBeforeAny(deliveryID, eventName, event)
	if err != nil {
		return g.handleError(deliveryID, eventName, event, err)
	}

	err = g.handleMetaEventAny(deliveryID, eventName, event)
	if err != nil {
		return g.handleError(deliveryID, eventName, event, err)
	}

	err = g.handleAfterAny(deliveryID, eventName, event)
	if err != nil {
		return g.handleError(deliveryID, eventName, event, err)
	}
	return nil
}
